"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = void 0;
const Tools_1 = require("./Tools");
class Entity {
    constructor(data, map, G) {
        this.moving = false;
        this.abilities = {};
        this.cooperative = false;
        this.apiercing = 0;
        this.armor = 0;
        this.avoidance = 0;
        this.blast = 0;
        this.breaks = 0;
        this.crit = 0;
        this.critdamage = 0;
        this.evasion = 0;
        this.lifesteal = 0;
        this.mcourage = 0;
        this.reflection = 0;
        this.resistance = 0;
        this.rpiercing = 0;
        this["1hp"] = false;
        this.aa = 0;
        this.achievements = [];
        this.cute = false;
        this.escapist = false;
        this.global = false;
        this.goldsteal = 0;
        this.humanoid = false;
        this.immune = false;
        this.operator = false;
        this.passive = false;
        this.poisonous = false;
        this.prefix = "";
        this.roam = false;
        this.spawns = [];
        this.special = false;
        this.stationary = false;
        this.supporter = false;
        this.trap = false;
        this.unlist = false;
        this.level = 1;
        this.s = {};
        this.G = G;
        this.max_hp = G.monsters[data.type].hp;
        this.max_mp = G.monsters[data.type].mp;
        this.map = map;
        for (const gKey in G.monsters[data.type]) {
            this[gKey] = G.monsters[data.type][gKey];
        }
        this.updateData(data);
    }
    updateData(data) {
        if (this.id !== undefined && this.id !== data.id)
            throw Error("The entity's ID does not match");
        for (const key in data)
            this[key] = data[key];
    }
    calculateDamageRange(defender) {
        if (defender["1hp"])
            return [1, 1];
        let baseDamage = this.attack;
        if (defender.s.cursed)
            baseDamage *= 1.2;
        if (defender.s.marked)
            baseDamage *= 1.1;
        if (this.damage_type == "physical")
            baseDamage *= Tools_1.Tools.damage_multiplier(defender.armor - this.apiercing);
        else if (this.damage_type == "magical")
            baseDamage *= Tools_1.Tools.damage_multiplier(defender.resistance - this.rpiercing);
        if (this.crit) {
            if (this.crit >= 100) {
                return [baseDamage * 0.9 * (2 + (this.critdamage / 100)), baseDamage * 1.1 * (2 + (this.critdamage / 100))];
            }
            else {
                return [baseDamage * 0.9, baseDamage * 1.1 * (2 + (this.critdamage / 100))];
            }
        }
        else {
            return [baseDamage * 0.9, baseDamage * 1.1];
        }
    }
    couldDieToProjectiles(projectiles, players, entities) {
        if (this.avoidance >= 100)
            return false;
        let incomingProjectileDamage = 0;
        for (const projectile of projectiles.values()) {
            if (!projectile.damage)
                continue;
            if (projectile.target !== this.id)
                continue;
            const attacker = players.get(projectile.attacker) || entities.get(projectile.attacker);
            if (!attacker)
                return true;
            if (attacker.damage_type == "physical" && this.evasion >= 100)
                continue;
            if (attacker.damage_type == "magical" && this.reflection >= 100)
                continue;
            const maximumDamage = attacker.calculateDamageRange(this)[1];
            incomingProjectileDamage += maximumDamage;
            if (incomingProjectileDamage >= this.hp)
                return true;
        }
        return false;
    }
    couldGiveCreditForKill(player) {
        if (this.target == undefined)
            return true;
        if (this.cooperative)
            return true;
        if (this.isAttackingPartyMember(player))
            return true;
        return false;
    }
    isAttackingPartyMember(player) {
        if (this.target == undefined)
            return false;
        if (this.isAttackingUs(player))
            return true;
        if (player?.partyData?.list.includes(this.target))
            return true;
        return false;
    }
    isAttackingUs(player) {
        return this.target == player.id;
    }
    isTauntable(by) {
        if (this.target == undefined)
            return true;
        if (this.isAttackingPartyMember(by))
            return true;
        const targetting = by.players.get(this.target);
        if (targetting && targetting.owner == by.owner)
            return true;
        return false;
    }
    willBurnToDeath() {
        if (this["1hp"])
            return false;
        if (this.lifesteal)
            return false;
        if (this.abilities?.self_healing)
            return false;
        if (!this.s.burned)
            return false;
        const burnTime = Math.max(0, (this.s.burned.ms - (this.G.conditions.burned.interval * 2))) / 1000;
        const burnDamage = burnTime * this.s.burned.intensity;
        if (burnDamage > this.hp)
            return true;
        return false;
    }
    willDieToProjectiles(character, projectiles, players, entities) {
        if (this.avoidance)
            return false;
        let incomingProjectileDamage = 0;
        for (const projectile of projectiles.values()) {
            if (!projectile.damage)
                continue;
            if (projectile.target !== this.id)
                continue;
            let attacker;
            if (!attacker && character.id == projectile.attacker)
                attacker = character;
            if (!attacker)
                attacker = players.get(projectile.attacker);
            if (!attacker)
                attacker = entities.get(projectile.attacker);
            if (!attacker)
                continue;
            if (attacker.damage_type == "magical" && this.reflection)
                continue;
            if (attacker.damage_type == "physical" && this.evasion)
                continue;
            const minimumDamage = attacker.calculateDamageRange(this)[0];
            incomingProjectileDamage += minimumDamage;
            if (incomingProjectileDamage >= this.hp)
                return true;
        }
        return false;
    }
}
exports.Entity = Entity;
//# sourceMappingURL=Entity.js.map