"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Game = void 0;
const axios_1 = __importDefault(require("axios"));
const fs_1 = __importDefault(require("fs"));
const Database_1 = require("./database/Database");
const Paladin_1 = require("./Paladin");
const Mage_1 = require("./Mage");
const Merchant_1 = require("./Merchant");
const Observer_1 = require("./Observer");
const PingCompensatedCharacter_1 = require("./PingCompensatedCharacter");
const Priest_1 = require("./Priest");
const Ranger_1 = require("./Ranger");
const Rogue_1 = require("./Rogue");
const Warrior_1 = require("./Warrior");
class Game {
    constructor() {
    }
    static async getGData(cache = false) {
        if (this.G)
            return this.G;
        if (!this.version)
            await this.getVersion();
        const gFile = `G_${this.version}.json`;
        try {
            this.G = JSON.parse(fs_1.default.readFileSync(gFile, "utf8"));
            return this.G;
        }
        catch (e) {
            console.debug("Updating 'G' data...");
            const response = await axios_1.default.get("http://adventure.land/data.js");
            if (response.status == 200) {
                const matches = response.data.match(/var\s+G\s*=\s*(\{.+\});/);
                const rawG = matches[1];
                this.G = JSON.parse(rawG);
                console.debug("Updated 'G' data!");
                if (cache)
                    fs_1.default.writeFileSync(gFile, rawG);
                return this.G;
            }
            else {
                console.error(response);
                console.error("Error fetching http://adventure.land/data.js");
            }
        }
    }
    static async getMail(all = true) {
        if (!this.user)
            return Promise.reject("You must login first.");
        let response = await axios_1.default.post("http://adventure.land/api/pull_mail", "method=pull_mail&arguments={}", { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
        const mail = [];
        while (response.data.length > 0) {
            mail.push(...response.data[0].mail);
            if (all && response.data[0].more) {
                response = await axios_1.default.post("http://adventure.land/api/pull_mail", `method=pull_mail&arguments={"cursor":"${response.data[0].cursor}"}`, { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
            }
            else {
                break;
            }
        }
        return mail;
    }
    static async getMerchants() {
        if (!this.user)
            return Promise.reject("You must login first.");
        const merchants = [];
        const data = await axios_1.default.post("http://adventure.land/api/pull_merchants", "method=pull_merchants", { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
        for (const datum of data.data) {
            if (datum.type == "merchants") {
                for (const char of datum.chars) {
                    merchants.push(char);
                }
            }
        }
        if (Database_1.Database.connection) {
            const informationDate = Date.now() - 300000;
            const promises = [];
            for (const merchant of merchants) {
                const server = merchant.server.split(" ");
                promises.push(Database_1.PlayerModel.updateOne({ lastSeen: { $lt: informationDate }, name: merchant.name }, {
                    lastSeen: informationDate,
                    map: merchant.map,
                    serverIdentifier: server[1],
                    serverRegion: server[0],
                    "slots.trade1": merchant.slots.trade1,
                    "slots.trade2": merchant.slots.trade2,
                    "slots.trade3": merchant.slots.trade3,
                    "slots.trade4": merchant.slots.trade4,
                    "slots.trade5": merchant.slots.trade5,
                    "slots.trade6": merchant.slots.trade6,
                    "slots.trade7": merchant.slots.trade7,
                    "slots.trade8": merchant.slots.trade8,
                    "slots.trade9": merchant.slots.trade9,
                    "slots.trade10": merchant.slots.trade10,
                    "slots.trade11": merchant.slots.trade11,
                    "slots.trade12": merchant.slots.trade12,
                    "slots.trade13": merchant.slots.trade13,
                    "slots.trade14": merchant.slots.trade14,
                    "slots.trade15": merchant.slots.trade15,
                    "slots.trade16": merchant.slots.trade16,
                    "slots.trade17": merchant.slots.trade17,
                    "slots.trade18": merchant.slots.trade18,
                    "slots.trade19": merchant.slots.trade19,
                    "slots.trade20": merchant.slots.trade20,
                    "slots.trade21": merchant.slots.trade21,
                    "slots.trade22": merchant.slots.trade22,
                    "slots.trade23": merchant.slots.trade23,
                    "slots.trade24": merchant.slots.trade24,
                    "slots.trade25": merchant.slots.trade25,
                    "slots.trade26": merchant.slots.trade26,
                    "slots.trade27": merchant.slots.trade27,
                    "slots.trade28": merchant.slots.trade28,
                    "slots.trade29": merchant.slots.trade29,
                    "slots.trade30": merchant.slots.trade30,
                    x: merchant.x,
                    y: merchant.y
                }).exec());
            }
            await Promise.all(promises);
        }
        return merchants;
    }
    static async getVersion() {
        const response = await axios_1.default.get("http://adventure.land/comm");
        if (response.status == 200) {
            const matches = response.data.match(/var\s+VERSION\s*=\s*'(\d+)/);
            this.version = Number.parseInt(matches[1]);
            return this.version;
        }
        else {
            console.error(response);
            console.error("Error fetching http://adventure.land/comm");
        }
    }
    static async markMailAsRead(mailID) {
        if (!this.user)
            return Promise.reject("You must login first.");
        const response = await axios_1.default.post("http://adventure.land/api/read_mail", `method=read_mail&arguments={"mail": "${mailID}"}`, { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
        return response.data[0];
    }
    static async login(email, password, mongo) {
        if (mongo)
            await Database_1.Database.connect(mongo);
        if (Database_1.Database.connection) {
            const find = await Database_1.AuthModel.findOne({ email: email }).lean().exec();
            if (find?.userID && find?.userAuth) {
                console.debug("Using auth data from database...");
                this.user = { userAuth: find.userAuth, userID: find.userID };
                return this.updateServersAndCharacters();
            }
        }
        console.debug("Logging in...");
        const login = await axios_1.default.post("https://adventure.land/api/signup_or_login", `method=signup_or_login&arguments={"email":"${email}","password":"${password}","only_login":true}`);
        let loginResult;
        for (const datum of login.data) {
            if (datum.message) {
                loginResult = datum;
                break;
            }
        }
        if (loginResult && loginResult.message == "Logged In!") {
            console.debug("Logged in!");
            for (const cookie of login.headers["set-cookie"]) {
                const result = /^auth=(.+?);/.exec(cookie);
                if (result) {
                    this.user = {
                        userAuth: result[1].split("-")[1],
                        userID: result[1].split("-")[0]
                    };
                    if (Database_1.Database.connection)
                        await Database_1.AuthModel.updateOne({ email: email }, { userAuth: this.user.userAuth, userID: this.user.userID }, { upsert: true }).lean().exec();
                    break;
                }
            }
        }
        else if (loginResult && loginResult.message) {
            console.error(loginResult.message);
            return Promise.reject(loginResult.message);
        }
        else {
            console.error(login.data);
            return Promise.reject("Failed logging in.");
        }
        return this.updateServersAndCharacters();
    }
    static async loginJSONFile(path) {
        const data = JSON.parse(fs_1.default.readFileSync(path, "utf8"));
        return this.login(data.email, data.password, data.mongo);
    }
    static async logoutEverywhere() {
        if (!this.user)
            return Promise.reject("You must login first.");
        const response = await axios_1.default.post("http://adventure.land/api/logout_everywhere", "method=logout_everywhere", { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
        if (Database_1.Database.connection)
            await Database_1.AuthModel.deleteOne({ userID: this.user.userID }).exec();
        this.user = undefined;
        return response.data;
    }
    static async startCharacter(cName, sRegion, sID, cType) {
        if (!this.user)
            return Promise.reject("You must login first.");
        if (!this.characters)
            await this.updateServersAndCharacters();
        if (!this.G)
            await this.getGData();
        const userID = this.user.userID;
        const userAuth = this.user.userAuth;
        if (!this.characters[cName])
            return Promise.reject(`You don't have a character with the name '${cName}'`);
        const characterID = this.characters[cName].id;
        try {
            let player;
            if (cType == "mage")
                player = new Mage_1.Mage(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "merchant")
                player = new Merchant_1.Merchant(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "paladin")
                player = new Paladin_1.Paladin(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "priest")
                player = new Priest_1.Priest(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "ranger")
                player = new Ranger_1.Ranger(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "rogue")
                player = new Rogue_1.Rogue(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else if (cType == "warrior")
                player = new Warrior_1.Warrior(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            else
                player = new PingCompensatedCharacter_1.PingCompensatedCharacter(userID, userAuth, characterID, Game.G, this.servers[sRegion][sID]);
            try {
                await player.connect();
            }
            catch (e) {
                return Promise.reject(e);
            }
            return player;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    static async startMage(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "mage");
    }
    static async startMerchant(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "merchant");
    }
    static async startPaladin(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "paladin");
    }
    static async startPriest(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "priest");
    }
    static async startRanger(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "ranger");
    }
    static async startRogue(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "rogue");
    }
    static async startWarrior(cName, sRegion, sID) {
        return await Game.startCharacter(cName, sRegion, sID, "warrior");
    }
    static async startObserver(region, id) {
        try {
            if (!this.G)
                await this.getGData();
            const observer = new Observer_1.Observer(this.servers[region][id], this.G);
            await observer.connect(true);
            return observer;
        }
        catch (e) {
            return Promise.reject(e);
        }
    }
    static async updateServersAndCharacters() {
        if (!this.user)
            return Promise.reject("You must login first.");
        const data = await axios_1.default.post("http://adventure.land/api/servers_and_characters", "method=servers_and_characters&arguments={}", { headers: { "cookie": `auth=${this.user.userID}-${this.user.userAuth}` } });
        if (data.status == 200) {
            for (const serverData of data.data[0].servers) {
                if (!this.servers[serverData.region])
                    this.servers[serverData.region] = {};
                this.servers[serverData.region][serverData.name] = serverData;
            }
            for (const characterData of data.data[0].characters) {
                this.characters[characterData.name] = characterData;
            }
            return Promise.resolve(true);
        }
        else {
            console.error(data);
        }
        return Promise.reject("Error fetching http://adventure.land/api/servers_and_characters");
    }
}
exports.Game = Game;
Game.servers = {};
Game.characters = {};
//# sourceMappingURL=Game.js.map