"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Player = void 0;
const Tools_1 = require("./Tools");
class Player {
    constructor(data, map, g) {
        this.armor = 0;
        this.apiercing = 0;
        this.avoidance = 0;
        this.crit = 0;
        this.critdamage = 0;
        this.evasion = 0;
        this.level = 1;
        this.reflection = 0;
        this.resistance = 0;
        this.rpiercing = 0;
        this.s = {};
        this.c = {};
        this.q = {};
        this.G = g;
        this.map = map;
        if (!data.npc)
            this.damage_type = this.G.classes[data.ctype].damage_type;
        this.updateData(data);
    }
    updateData(data) {
        if (this.id !== undefined && this.id !== data.id)
            throw Error("The entity's ID does not match");
        for (const key in data)
            this[key] = data[key];
    }
    calculateDamageRange(defender, skill = "attack") {
        if (defender.immune && ["3shot", "5shot", "burst", "cburst", "supershot", "taunt"].includes(skill))
            return [0, 0];
        if (defender["1hp"] || skill == "taunt")
            return [1, 1];
        let baseDamage = this.attack;
        if (this.G.skills[skill].damage)
            baseDamage = this.G.skills[skill].damage;
        if (defender.s.cursed)
            baseDamage *= 1.2;
        if (defender.s.marked)
            baseDamage *= 1.1;
        if (this.ctype == "priest")
            baseDamage *= 0.4;
        let additonalApiercing = 0;
        if (this.G.skills[skill].apiercing)
            additonalApiercing = this.G.skills[skill].apiercing;
        if (this.damage_type == "physical")
            baseDamage *= Tools_1.Tools.damage_multiplier(defender.armor - this.apiercing - additonalApiercing);
        else if (this.damage_type == "magical")
            baseDamage *= Tools_1.Tools.damage_multiplier(defender.resistance - this.rpiercing);
        if (this.G.skills[skill].damage_multiplier)
            baseDamage *= this.G.skills[skill].damage_multiplier;
        let lowerLimit = baseDamage * 0.9;
        let upperLimit = baseDamage * 1.1;
        if (this.crit) {
            if (this.crit >= 100) {
                lowerLimit *= (2 + (this.critdamage / 100));
            }
            upperLimit *= (2 + (this.critdamage / 100));
        }
        if (skill == "cleave") {
            lowerLimit *= 0.1;
            upperLimit *= 0.9;
        }
        return [lowerLimit, upperLimit];
    }
    isFriendly(bot) {
        if (bot.owner == this.owner)
            return true;
        if (bot.party == this.party)
            return true;
        return false;
    }
    isNPC() {
        return this.npc !== undefined;
    }
}
exports.Player = Player;
//# sourceMappingURL=Player.js.map